#!/usr/bin/env python
# -*- coding: utf-8 -*-
        
#  Copyright 2008 Peter Mora                                                                                                                             
#
#  This file is part of Vakarcs.
#
#  Vakarcs is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Vakarcs is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
#   

import lang
import random
MAP_EMPTY = 1
MAP_WALL = 2
MAP_DESTROYED = 4
MAP_VISIBLE = 8
MAP_CLOAKED = 16
#MAP_BASE = 16 # for player p it's 16*2^p


class Map:
    def __init__(self,map=[[]],shop=[]):
        self.map = [[y for y in x] for x in map]
        self.objects = []
        self.active_step = 0
        self.round_number = 0
        self.message = ""
        self.autoshop = 0
        self.shop = shop
    def size(self):
        return (len(self.map),len(self.map[0]))

class StackEvent:
    def __init__(self,time,function,pos,args,watchers,server):
        self.time = time
        self.function = function
        self.pos = pos
        self.args = args
        self.watchers = watchers
        self.server = server
    def run(self):
        getattr(self.server,self.function)(self.pos,self.watchers,*self.args)


class Ability:
    def __init__(self,name,type,data):
        self.name = name
        self.type = type # "dir", "pos" or "shop"
        self.data = data # list

class Object:
    def __init__(self,owner,type,pos,server,abilities = [], prop = {},shop = None):
        self.owner = owner
        self.type = type
        self.pos = pos
        self.step = {}
        self.prop = {"life":1}
        self.abilities = abilities
        if shop != None:
            #print type
            self.abilities = [s[2] for s in shop if s[1]==type][0]
            self.prop = [s[3] for s in shop if s[1]==type][0]
        for p in prop:
            self.prop[p] = prop[p]
        self.future_steps = ["" for i in range(server.number_of_steps)]
        self.id = server.object_id
        server.object_id += 1
        self.color = [0,0,0]
    def property(self,p):
        if p in self.prop:
            return self.prop[p]
        return 0
    def shootdir(self,data,s,server):
        if s==60: #shooting
            dir = eval(data)
            new_pos = [self.pos[0]+(dir==0)-(dir==2),self.pos[1]+(dir==3)-(dir==1)]
            while server.in_map(new_pos) and server.active_map(new_pos) & MAP_WALL == 0:
                for pid in server.who_can_see_it(self.pos, self.owner):
                    server.player_map_add(id=pid, pos=new_pos, what=MAP_DESTROYED)
                [server.stack.append(StackEvent(time=[0,0],function="destroy_object",pos=[],args=[ob],watchers=server.who_can_see_it(ob.pos,ob.owner,self.owner),server=server)) for ob in server.map.objects if ob.pos == new_pos]
                new_pos = [new_pos[0]+(dir==0)-(dir==2),new_pos[1]+(dir==3)-(dir==1)]
            if len([s for s in self.abilities if s.name == "move"]):
                server.players[self.owner].needs_map = 1
                server.players[self.owner].player_map.message = u"A vakarcsod többet nem tud mozogni."
                self.abilities = [s for s in self.abilities if s.name != "move"]

    def shootpos(self,data,s,server):
        if s==60: #shooting
            new_pos = list(eval(data))
            if server.in_map(new_pos):
                server.stack.append(StackEvent(time=[5,60],function="destroy_object_if_visible", pos=new_pos, args=[self], watchers=range(server.number_of_players), server=server))
                for id in range(server.number_of_players):
                        server.players[id].needs_map = 1
        
    def move(self,data,s,server):
        if s==20: #moving
            dir = eval(data)
            new_pos = [self.pos[0]+(dir==0)-(dir==2),self.pos[1]+(dir==3)-(dir==1)]
            if server.in_map(new_pos) and server.active_map(new_pos) & MAP_WALL == 0:
                self.pos = new_pos
            else:
                server.players[self.owner].needs_map = 1
                server.players[self.owner].player_map.message = u"A vakarcsod nem tudott lépni."

    def crazymove(self,server):
        for i in range(self.prop["crazymove"]):
            new_pos = [-1,-1]
            while server.in_map(new_pos) == False:
                dir = random.choice([0,1,2,3])
                new_pos = [self.pos[0]+(dir==0)-(dir==2),self.pos[1]+(dir==3)-(dir==1)]
            if server.active_map(new_pos) & MAP_WALL == 0:
                self.pos = new_pos

    def distsee(self,data,s,server):
        if s==40: #seeing
            tempmap = [[0 for y in x] for x in server.map.map]
            tempmap[self.pos[0]][self.pos[1]] = 0
            old = [[self.pos[0],self.pos[1]]]
            new = []
            i = 0
            m = [a.data[0] for a in self.abilities if a.name == "distsee"][0]
            while old != [] and i<m:
                i += 1
                while old != []:
                    p = old[0]
                    old[0:1] = []
                    for pos in [ [p[0]-1,p[1]], [p[0]+1,p[1]], [p[0],p[1]+1], [p[0],p[1]-1] ]:
                        if server.in_map(pos) and tempmap[pos[0]][pos[1]] == 0:
                            if server.active_map(pos) & MAP_WALL == 0:
                                new.append([pos[0],pos[1]])
                            tempmap[pos[0]][pos[1]] = 1
                            server.players[self.owner].player_visible[pos[0]][pos[1]] = 1
                old = new
                new = []

    def cloak(self,data,s,server):
        if s == 10: # cloak
            server.active_map_add(pos=self.pos, what=MAP_CLOAKED)

    def build(self,data,s,server):
        if s == 92: # wall build
            dir = eval(data)
            new_pos = [self.pos[0]+(dir==0)-(dir==2),self.pos[1]+(dir==3)-(dir==1)]
            if len([o for o in server.map.objects if o.pos == new_pos]):
                server.players[self.owner].needs_map = 1
                server.players[self.owner].player_map.message = u"Falépítés nem sikerül, áll ott valaki."
            else:
                server.map.map[new_pos[0]][new_pos[1]] |= MAP_WALL
                for id in server.who_can_see_it(new_pos,self.owner):
                    server.players[id].needs_map = 1

    def destroy(self,data,s,server):
        if s == 91: # wall destroy
            dir = eval(data)
            new_pos = [self.pos[0]+(dir==0)-(dir==2),self.pos[1]+(dir==3)-(dir==1)]
            if server.active_map(new_pos) & MAP_WALL:
                server.active_map_remove(pos=new_pos, what=MAP_WALL)
                for id in server.who_can_see_it(new_pos,self.owner):
                    server.players[id].needs_map = 1

    def create(self,data,s,server):
        if s==90: #create
            ab = [a for a in self.abilities if a.name == "create"][0]
            d = [a for a in ab.data if a[1] == data][0]
            server.players[self.owner].needs_map = 1
            if server.players[self.owner].money < d[0]:
                server.players[self.owner].player_map.message = u"Nincs elég pénzed!"
            else:
                server.players[self.owner].money -= d[0]
                server.map.objects.append(Object(self.owner,d[1],self.pos,server,d[2],d[3]))
                server.players[self.owner].player_map.message = ""u"Vásároltál egy " + lang.type(d[1]) + u" vakarcsot."
    def line(self,ff,tt):
        f,t = ff[:],tt[:]
        if t==f:
            return [t]
        if abs(t[0]-f[0])>abs(t[1]-f[1]):
            if t[0] < f[0]:
                t,f = f,t
            return [ [x, f[1]+(t[1]-f[1])*(x-f[0])/(t[0]-f[0])] for x in range(f[0],t[0]+1) ] +\
                [ [x, f[1]+((t[1]-f[1])*(x-f[0])+t[0]-f[0]-1)/(t[0]-f[0])] for x in range(f[0],t[0]+1) ]
        else:
            if t[1] < f[1]:
                t,f = f,t
            return [ [f[0]+(t[0]-f[0])*(y-f[1])/(t[1]-f[1]),y] for y in range(f[1],t[1]+1) ] +\
                [ [f[0]+((t[0]-f[0])*(y-f[1])+t[1]-f[1]-1)/(t[1]-f[1]),y] for y in range(f[1],t[1]+1) ]

    def see(self,server):
        dist = 3
        if "seedist" in self.prop:
            dist = self.prop["seedist"]
        value = 1
        if "truesight" in self.prop:
            value = 2
        for x in range(-dist,dist+1):
            for y in range(-dist,dist+1):
                new_pos = [self.pos[0]+x,self.pos[1]+y]
                if server.in_map(new_pos) and (new_pos[0]-self.pos[0])**2+(new_pos[1]-self.pos[1])**2 <= dist**2:
                    if all([server.map.map[p[0]][p[1]] & (MAP_WALL | MAP_CLOAKED) == 0 or p==new_pos for p in self.line(self.pos,new_pos) ]):
                        server.players[self.owner].player_visible[new_pos[0]][new_pos[1]] = max(value,server.players[self.owner].player_visible[new_pos[0]][new_pos[1]])
                    #print self.pos, new_pos, [p for p in self.line(self.pos,new_pos) ]

    def make_step(self,s,server,active_step):
        fs = self.future_steps[active_step]
        if 0<fs.find('(')>0 < fs.find(')')  and hasattr(self,fs[:fs.find('(')]) and fs[:fs.find('(')] in [a.name for a in self.abilities]:
            getattr(self, fs[:fs.find('(')])(data=fs[fs.find('(')+1:fs.find(')')],s=s,server=server)
        if s==20 and "crazymove" in self.prop: #crazymoving
            self.crazymove(server)
        if s == 40: #seeing
            self.see(server)
        if s == 90: #autoshop
            if self.type == server.shopable and 0<=server.autoshop[self.owner] < len(server.shop):
                d = server.shop[server.autoshop[self.owner]]
                if server.players[self.owner].money >= d[0]:
                    server.players[self.owner].money -= d[0]
                    server.map.objects.append(Object(self.owner,d[1],self.pos,server,d[2],d[3]))
                    server.players[self.owner].player_map.message = ""u"Vásároltál egy " + lang.type(d[1]) + u" vakarcsot."
                    server.players[self.owner].needs_map = 1

