#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright 2008 Peter Mora                                                                                                                             
#
#  This file is part of Vakarcs.
#
#  Vakarcs is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Vakarcs is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
#   

from PodSixNet.Connection import *
from PodSixNet.Server import Server
from PodSixNet.Channel import Channel

from weakref import WeakKeyDictionary
import copy
import pickle

from object import *
from random import random
from math import sqrt


COLORS = [[8,5,151], [136,11,25], [167,147,14], [1,150,255]]

class Player:
    def __init__(self,id,cp=None):
        self.id = id
        self.color = [0,0,0]
        self.needs_map = 0
        self.message = ""
        self.stack = []
        self.winners = []
        self.rate = 0.5
        if cp != None:
            self.money = cp.money
            self.rate = cp.rate
            self.money_per_round = cp.money_per_round
            self.player_map = copy.deepcopy(cp.player_map)
            self.color = cp.color[:]

class VakarcsServer:
    def __init__(self,map_file,comm_server):
        self.map = Map()
        self.comm_server = comm_server
        self.object_id = 0
        execfile(map_file+".py") #Reading map file
        self.comm_server.number_of_steps = self.number_of_steps
        self.players = [Player(id=i) for i in range(self.number_of_players)]
        self.last_round = -1
        self.winners = []
        self.autoshop = [0 for i in range(self.number_of_players)]
        for p in self.players:
            p.player_map = Map(self.map.map,self.shop)
            p.needs_map = 0
            p.player_visible = [[0 for y in range(self.map.size()[1])] for x in range(self.map.size()[0])]
            p.money = self.start_money
            p.money_per_round = self.money_per_round
            if p.id < len(COLORS):
                p.color = COLORS[p.id]
            else:
                p.color = [int(255*random()) for i in [0,1,2]]
        self.mx = self.map.size()[0]
        self.my = self.map.size()[1]
        self.number_of_next_steps = 1
        self.stack = []
    def cx(self,x):
        return min(self.map.size()[0]-1,max(0,x))
    def cy(self,y):
        return min(self.map.size()[1]-1,max(0,y))


    def in_map(self,pos):
        return 0<= pos[0] < self.map.size()[0] and 0<= pos[1] < self.map.size()[1]

    def active_map(self,pos):
        return self.map.map[pos[0]][pos[1]]

    def active_map_add(self,pos,what):
        self.map.map[pos[0]][pos[1]] |= what

    def active_map_remove(self,pos,what):
        self.map.map[pos[0]][pos[1]] = self.map.map[pos[0]][pos[1]] & (self.map.map[pos[0]][pos[1]] ^ what)

    def player_map_add(self,id,pos,what):
        self.players[id].player_map.map[pos[0]][pos[1]] |= what
        self.players[id].needs_map = 1


    def who_can_see_it(self,pos,owner=None,owner2=None):
        s = []
        if pos != []:
            s += [i for i in range(self.number_of_players) if self.players[i].player_map.map[pos[0]][pos[1]] & MAP_VISIBLE]
        if owner != None:
            s += [owner]
        if owner2 != None:
            s += [owner2]
        return list(set(s))


    def generate_map(self,p,step):#,message=""):
        for x in range(self.mx):
            for y in range(self.my):
                if p.player_visible[x][y] > 0:
                    p.player_map.map[x][y] = self.map.map[x][y] | MAP_VISIBLE | (p.player_map.map[x][y] & MAP_DESTROYED)
              #      if p.player_map.map[x][y] & MAP_CLOAKED:
              #          p.player_map.map[x][y] = (p.player_map.map[x][y] ^ MAP_CLOAKED) | MAP_WALL
                        #print x,y
                else:
                    p.player_map.map[x][y] = p.player_map.map[x][y] & (p.player_map.map[x][y] ^ MAP_DESTROYED)

        p.player_map.objects=[]
        for ob in self.map.objects:
            ob.color = self.players[ob.owner].color
            if ob.owner == p.id or (p.player_visible[ob.pos[0]][ob.pos[1]] >= 1 and self.active_map(ob.pos) & MAP_CLOAKED ==0)\
                        or p.player_visible[ob.pos[0]][ob.pos[1]] >= 2:
                p.player_map.objects.append(copy.deepcopy(ob))
                if step == 0:
                    p.player_map.objects[-1].future_steps =  ["" for i in range(self.number_of_steps)]
        p.player_map.round_number = self.last_round
        p.player_map.active_step = step
        p.player_map.stack = [ [s.time,s.function,s.pos,s.args] for s in self.stack]
        p.player_map.winners = self.winners
        self.comm_server.sync_data[p.id].append(Player(id=p.id, cp=p))
        p.player_map.message = ""
        p.player_map.map = [[y & (y^MAP_DESTROYED)  for y in x ] for x in p.player_map.map]

    def make_round(self):
        # read the users' steps
        for p in self.comm_server.players:
            if p.map != None:
                for pob in p.map.objects:
                    for ob in self.map.objects:
                        if ob.id == pob.id and p.id == ob.owner:
                            ob.future_steps = pob.future_steps
                    self.autoshop[p.id] = p.map.autoshop
                if self.comm_server.sync_data[p.id] != []: # update sync_data with the users' steps
                    self.comm_server.sync_data[p.id][-1].player_map.objects = p.map.objects
        for step in range(self.number_of_next_steps):
            self.make_step(step) # make the steps
        self.number_of_next_steps = self.number_of_steps

    def dist(self,pos1,pos2):
        return sqrt((pos1[0]-pos2[0])**2 + (pos1[1]-pos2[1])**2)

    def make_step(self,step):
        if len(self.winners):
            return
        self.step = step
        self.last_round += 1
        # clear player_visible
        for p in self.players:
            p.player_visible = [[0 for y in range(self.map.size()[1])] for x in range (self.map.size()[0])]
            p.player_map.map = [[y & (y^MAP_VISIBLE)  for y in x ] for x in p.player_map.map]
            p.player_map.objects = []
        # call objects
        for s in range(100):
            if len(self.winners):
                return
            for ob in self.map.objects:
                ob.make_step(s,self,step)
            if s == 40:
                for p in self.players:
                    p.needs_map = 1
            self.winners = self.check_end(self) 
            if len(self.winners)>0:
                [ self.generate_map(p,(step+1)%self.number_of_next_steps) for p in self.players]
                return
            [ self.generate_map(p,(step+1)%self.number_of_next_steps) for p in self.players if p.needs_map ]
            for p in self.players:
                p.needs_map = 0
            for st in self.stack: # run stack
                if st.time[0] == 0 and st.time[1] <= s:
                    self.stack.remove(st)
                    st.run()
            self.winners = self.check_end(self) 
            if len(self.winners)>0:
                [ self.generate_map(p,(step+1)%self.number_of_next_steps) for p in self.players]
                return
            [ self.generate_map(p,(step+1)%self.number_of_next_steps) for p in self.players if p.needs_map ]
            for p in self.players:
                p.needs_map = 0
        for st in self.stack: # decrease time in stack
            st.time[0] -= 1
        for x in range(self.mx):
            for y in range(self.my):
                self.map.map[x][y] = self.map.map[x][y] & ( self.map.map[x][y] ^ MAP_CLOAKED)
        # add money
        for p in self.players:
            rate = 0
            for x in range(self.map.size()[0]):
                for y in range(self.map.size()[1]):
                    if any([all([self.dist(o.pos,[x,y]) < self.dist(other.pos,[x,y]) for other in self.map.objects if other.owner!=p.id]) for o in self.map.objects if o.owner==p.id]):
                        rate += 1.0/self.map.size()[0]/self.map.size()[1]
            #print rate
            p.money += self.money_per_round*rate
            p.rate = rate

    def destroy_object(self,pos,watchers,ob):
        if ob in self.map.objects:
            if ob.property("life") <= 1:
                self.map.objects.remove(ob)
                #print "removed"
            else:
                #print "losing life"
                ob.prop["life"] = ob.property("life")-1
            for pid in watchers:
                self.players[pid].player_map.message = lang.shot(ob.type)
                self.generate_map(self.players[pid],(self.step+1)%self.number_of_next_steps)
    def destroy_object_if_visible(self,pos,watchers,own_ob):
        if self.players[own_ob.owner].player_map.map[pos[0]][pos[1]] & MAP_VISIBLE:
            for pid in watchers:
                self.players[pid].player_map.map[pos[0]][pos[1]] |= MAP_DESTROYED
            for ob in self.map.objects:
                if ob.pos == pos:
                    self.destroy_object(pos=[], watchers=range(self.number_of_players), ob=ob)

class ServerChannel(Channel):
    """
    This is a server-side representation of a single connected client.
    """
    def __init__(self, *args, **kwargs):
        Channel.__init__(self, *args, **kwargs)
        self.id = self._server.NextId()
        intid = int(self.id)
        self.nickname = "anonymous"
        self.message = ""
        self.ready = False
        self.map = None
    
#    def PassOn(self, data):
        # pass on what we received to all connected clients
#        data.update({"id": self.id})
#        self._server.SendToAll(data)
#        pass


    
    def Close(self):
        self._server.DelPlayer(self)
    
    ##################################
    ### Network specific callbacks ###
    ##################################
#    def Network_startline(self, data):
#        self.PassOn(data)
#    
#    def Network_drawpoint(self, data):
#        self.PassOn(data)

    def Network_nickname(self,data):
        self.nickname = data['nickname']
        self.message = data['message']
        self.ready = data['ready']
        self._server.SendPlayers()

    def Network_map(self,data):
        self.map = pickle.loads(data['map'])

class ConnectionServer(Server):
    channelClass = ServerChannel
    
    def __init__(self, *args, **kwargs):
        self.id = []
        Server.__init__(self, *args, **kwargs)
        self.players = WeakKeyDictionary()
        self.vakarcs_server = None
        self.started = False
        #print 'Server launched'
    
    def NextId(self):
        r = 0
        while r in self.id:
            r += 1
        self.id.append(r)
        self.id.sort()
        return r
    
    def Connected(self, channel, addr):
        if self.started == False:
            self.AddPlayer(channel)
            if self.vakarcs_server != None and self.vakarcs_server.number_of_players == len(self.id):
                self.started = True
                self.sync_data = [[] for p in range(self.vakarcs_server.number_of_players)]
                self.vakarcs_server.make_round()
                self.Sync()
    
    def AddPlayer(self, player):
        print "New Player" + str(player.addr)
        self.players[player] = True
        player.Send({"action": "initial","id":player.id})
        self.SendPlayers()
    
    def DelPlayer(self, player):
        print "Deleting Player" + str(player.addr)
        del self.players[player]
        self.SendPlayers()
    
    def SendPlayers(self):
        #print "SendPlayers"
        self.SendToAll({"action": "nicknames", "nicknames": [p.nickname for p in self.players], "messages": [p.message for p in self.players], "ids": [p.id for p in self.players], "readys": [p.ready for p in self.players]})
        if [p.ready for p in self.players] == [True] * self.vakarcs_server.number_of_players:
            self.Pump()
            self.vakarcs_server.make_round()
            self.Sync()
            for p in self.players:
                p.ready = False
            self.SendPlayers()
    
    def SendToAll(self, data):
        #print "SendToAll"
        [p.Send(data) for p in self.players]

    def Sync(self):
        [p.Send({"action": "sync", "sync": pickle.dumps(self.sync_data[p.id]), "number_of_steps" : self.number_of_steps}) for p in self.players]
    

class Client(ConnectionListener):
    def __init__(self,server_name, name,ui):
        self.Connect((server_name, 31425))
        connection.Send({"action":"nickname","nickname": name,"message":"","ready":0})
        self.players = []
        self.messages = []
        self.readys = []
        self.ids = []
        self.id = ""
        self.name = name
        self.message = ""
        self.ui = ui
        self.sync_data = []
        self.ready = 0

    def SendNickMess(self):
        connection.Send({"action":"nickname","nickname": self.name,"message":self.message,"ready":self.ready})
    
    def SendMap(self):
        self.ui.maps[-1].autoshop = self.ui.autoshop
        connection.Send({"action":"map","map": pickle.dumps(self.ui.maps[-1])})

    def Loop(self):
        self.Pump()
        connection.Pump()
#        self.Events()
#        self.Draw([(self.players[p]['color'], self.players[p]['lines']) for p in self.players])
#
#        if "connecting" in self.statusLabel:
#            self.statusLabel = "connecting" + ("." * ((self.frame / 30) % 4))

    ####################### 
    ### Event callbacks ###
    #######################
    #def PenDraw(self, e):
    #   connection.Send({"action": "draw", "point": e.pos})

    ###############################
    ### Network event callbacks ###
    ###############################

    def Network_initial(self, data):
        self.id = data['id']
        pass

    def Network_nicknames(self, data):
        self.players = data['nicknames']
        self.messages = data['messages']
        self.readys = data['readys']
        self.ids = data['ids']
        self.ui.redraw = True
        pass


    def Network_sync(self,data):
        self.sync_data = pickle.loads(data['sync'])
        self.ui.player_info = self.sync_data
        self.ui.maps = [s.player_map for s in self.sync_data]
        self.ui.number_of_steps = data['number_of_steps']
#        print [ob.pos for ob in self.ui.map.objects]
        self.ready = 0
        self.ui.sync()

    def Network(self, data):
        #print 'network:', data
        pass
    
    def Network_connected(self, data):
        self.statusLabel = "connected"
        self.ui.redraw = True
    
    def Network_error(self, data):
        #print data
        import traceback
        traceback.print_exc()
        self.statusLabel = data['error'][1]
        connection.Close()
        self.ui.redraw = True
    
    def Network_disconnected(self, data):
        self.statusLabel += " - disconnected"
        print "connection lost"
        self.ui.redraw = True

