#!/usr/bin/env python
# -*- coding: utf-8 -*-
        
#  Copyright 2008 Peter Mora                                                                                                                             
#
#  This file is part of Vakarcs.
#
#  Vakarcs is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Vakarcs is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
#   

import sys, os

import pygame

from pygame.locals import *

import random, time

from object import *
from server import *

import lang


clock = pygame.time.Clock()

SCREENSIZE = (790, 480)
BLOCK_SIZE=20
NORMAL_MODE = 0
ABILITY_MODE = 1
MOVIE_MODE = 2
ABILITY_DIR = 0
ABILITY_POS = 1
ABILITY_SHOP = 2
FPS = 10
os.environ['SDL_VIDEO_CENTERED'] = '1'
pygame.init()                                                                   
screen = pygame.display.set_mode(SCREENSIZE)
pygame.display.set_caption('Vakarcs')
bigfont = pygame.font.Font(None, 70)
middlefont = pygame.font.Font(None, 46)
font = pygame.font.Font(None, 20)
#pygame.mouse.set_visible(0)

def load_image(name, colorkey=None,color=None):
    fullname = os.path.join('images', name)
    try:
        image = pygame.image.load(fullname)
        if color not in [None,"None"]:
            pxarray = pygame.PixelArray(image);
            pxarray.replace((0,0,0),color);
            image = pxarray.surface.convert_alpha()
        else:
            image = image.convert_alpha()
    except pygame.error, message:
        print 'Cannot load image:', name
        raise SystemExit, message
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

background = pygame.Surface(screen.get_size())
background_image = load_image("background.png")
for x in range(0,screen.get_size()[0],background_image[1].width):
    for y in range(0,screen.get_size()[1],background_image[1].height):
        background.blit( background_image[0], (x,y) )

wall_image,wall_rect = load_image("wall.png")
fog_image,fog_rect = load_image("fog.png")
fire_image,fog_rect = load_image("fire.png")
objects_image = {}

def input_text(question,start="",only_one_key=False):
    answer=start
    redraw = True
    while 1:
        if redraw:
            screen.blit(background, (0, 0))
            text = bigfont.render(question+answer, 1, (10, 10, 10))
            textpos = text.get_rect(centerx=background.get_width()/2,centery=background.get_height()/2)
            screen.blit(text, textpos)
            pygame.display.flip()
            redraw = False
        for event in pygame.event.get():
            if event.type in [KEYDOWN,MOUSEBUTTONDOWN]:
                redraw = True
            if event.type == QUIT:
                sys.exit(0)
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                return ""
            elif event.type == KEYDOWN and event.key == K_RETURN:
                return answer
            elif event.type == KEYDOWN and event.key == K_BACKSPACE:
                answer = answer[:-1]
            elif event.type == KEYDOWN and len(pygame.key.name(event.key))==1:
                if pygame.key.get_mods() & KMOD_SHIFT:
                    answer += pygame.key.name(event.key).upper()
                else:
                    answer += pygame.key.name(event.key)
                if only_one_key:
                    return answer
        clock.tick(FPS)

def menu_choose(main_question,menu_list):
    r_list = []
    active_menu = 0
    redraw = True
    while 1:
        if redraw:
            screen.blit(background, (0, 0))
            text = bigfont.render(main_question, 1, (10, 10, 10))
            textpos = text.get_rect(centerx=background.get_width()/2,centery=(1)*screen.get_size()[1]/(len(menu_list)+2))
            screen.blit(text, textpos)
            r_list = []
            for i in range(len(menu_list)):
                text = middlefont.render(menu_list[i], 1, (10+(i==active_menu)*180, 10, 10))
                textpos = text.get_rect(centerx=background.get_width()/2,centery=(i+2)*screen.get_size()[1]/(len(menu_list)+2))
                r_list.append(textpos)
                screen.blit(text, textpos)
            pygame.display.flip()
            redraw = False
        try:
            new_menu =  [r.collidepoint(pygame.mouse.get_pos()) for r in r_list].index(True)
            if active_menu != new_menu:
                active_menu = new_menu
                redraw = True
        except ValueError:
            pass
        for event in pygame.event.get():
            if event.type in [KEYDOWN,MOUSEBUTTONDOWN]:
                redraw = True
            if event.type == QUIT:
                sys.exit(0)
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                return -1
            elif event.type == KEYDOWN and event.key == K_DOWN:
                active_menu = (active_menu+1)%len(menu_list)
            elif event.type == KEYDOWN and event.key == K_UP:
                active_menu = (active_menu-1)%len(menu_list)
            elif event.type == KEYDOWN and event.key == K_RETURN:
                return active_menu
            elif event.type == MOUSEBUTTONDOWN and event.button == 1:
                try:
                    return [r.collidepoint(event.pos) for r in r_list].index(True)
                except ValueError:
                    pass
        clock.tick(FPS)

def load_object_image(image_name,size=0,color=None):
    if image_name+"size"+str(size)+"color"+str(color) not in objects_image:
        if size == 0:
            objects_image[image_name+"size"+str(size)+"color"+str(color) ] = load_image(image_name+".png",color=color)[0]
        else:
            objects_image[image_name+"size"+str(size)+"color"+str(color) ] = pygame.transform.smoothscale(load_image(image_name+".png",color=color)[0],(BLOCK_SIZE/size,BLOCK_SIZE/size))
    return objects_image[image_name+"size"+str(size)+"color"+str(color)]



class Ui:
    def __init__(self,name,server_name=None,map=None):
        self.name = name
        self.background = pygame.Surface(screen.get_size())                                                                                                                                                                                                                                       
        background_image = load_image("background2.png")
        for x in range(0,screen.get_size()[0],background_image[1].width):
                for y in range(0,screen.get_size()[1],background_image[1].height):
                            self.background.blit( background_image[0], (x,y) )

        if server_name == None:
            print "server"
            self.server = ConnectionServer(localaddr=("0.0.0.0", 31425))
            self.server.vakarcs_server = VakarcsServer(map,self.server)
            self.client = Client("localhost",name,self)
        else:
            print "client"
            self.server = None
            self.client = Client(server_name,name,self)
        
        self.maps = []
        self.redraw = True
        self.chosen_object = 0
        self.current_round = 0
        self.status_line = ""
        self.number_of_steps = 0
        self.bounding_boxes = []
        self.player_info = None
        self.mode = NORMAL_MODE
        self.ability_mode = 0
        self.ability_data = 0
        self.selected_ability = None
        self.last_maps_size = 0
        self.movie_count = 0
        self.autoshop = 0

    def check(self,sel = False, own = False, past_change = False,too_many_steps = False, status = True):
        if self.maps == []:
            if status: self.status_line = u"Még nem csatlakozott elegendő játékos."
            return True
        if past_change and self.current_round < len(self.maps) -1:
            if status: self.status_line = u"A múlton nem változtathatsz."
            return True
        map = self.maps[min(self.current_round,len(self.maps)-1)]
        selected_object = [o for o in map.objects if o.id == self.chosen_object]
        if sel and selected_object == []:
            if status: self.status_line = u"Először a TAB billentyű segítségével válassz ki egy vakarcsot!"
            return True
        if own and sel and selected_object[0].owner != self.client.id:
            if status: self.status_line = u"Csak a saját vakarcsaidat irányíthatod."
            return True
        if too_many_steps and self.current_round > len(self.maps) + self.number_of_steps -2:
            if status: self.status_line = u"Több lépést nem adhatsz meg!"
            return True
        return False

    def active_step(self):
        if self.current_round < len(self.maps)-1:
            return self.maps[self.current_round].active_step
        else:
            return self.current_round - len(self.maps) + 1
    def active_pos(self,ob):
        if self.current_round < len(self.maps):
            return ob.pos
        pos = ob.pos
        steps = ob.future_steps[:self.current_round - len(self.maps) + 1]
        pos = [pos[0]+len([s for s in steps if s == "move(0)"])-len([s for s in steps if s == "move(2)"]),\
                pos[1]-len([s for s in steps if s == "move(1)"])+len([s for s in steps if s == "move(3)"])]
        return pos

    def draw_map(self,map):
        self.bounding_boxes = []
        size = map.size()
        def p(ob):
            return map.size()[0]*self.active_pos(ob)[1]+self.active_pos(ob)[0]
        obs = {}
        for ob in map.objects:
            if p(ob) in obs:
                obs[p(ob)] += 1
            else:
                obs[p(ob)] = 1
        maxobs = {}
        for o in obs:
            if obs[o] > 9:
                maxobs[o] = 4
            else:
                maxobs[o] = [1,1,2,2,2,3,3,3,3,3][obs[o]]
        def posx(ob):
            return self.active_pos(ob)[0]*BLOCK_SIZE+int((obs[p(ob)] % maxobs[p(ob)])*float(BLOCK_SIZE)/maxobs[p(ob)])
        def posy(ob):
            return self.active_pos(ob)[1]*BLOCK_SIZE+int((obs[p(ob)] / maxobs[p(ob)])*float(BLOCK_SIZE)/maxobs[p(ob)])
        for x in range(size[0]):
            for y in range(size[1]):
                if map.map[x][y] & MAP_WALL:
                    screen.blit(wall_image, (x*BLOCK_SIZE, y*BLOCK_SIZE))
        for ob in map.objects:
            obs[p(ob)] -= 1
            screen.blit(load_object_image(ob.type,maxobs[p(ob)],color = ob.color),(posx(ob),posy(ob)))
            self.bounding_boxes.append([pygame.Rect((posx(ob),posy(ob)),(BLOCK_SIZE/maxobs[p(ob)],BLOCK_SIZE/maxobs[p(ob)])),ob.id])
            if ob.id == self.chosen_object:
                screen.blit(load_object_image("select"+str(maxobs[p(ob)])),(posx(ob),posy(ob)))
        for x in range(size[0]):
            for y in range(size[1]):
                if map.map[x][y] & MAP_VISIBLE == 0:
                    screen.blit(fog_image, (x*BLOCK_SIZE, y*BLOCK_SIZE))
                if map.map[x][y] & MAP_DESTROYED:
                    screen.blit(fire_image, (x*BLOCK_SIZE, y*BLOCK_SIZE))
                if map.map[x][y] & MAP_CLOAKED:
                    screen.blit(wall_image, (x*BLOCK_SIZE, y*BLOCK_SIZE))
        if self.mode == ABILITY_MODE and self.ability_mode == ABILITY_POS:
            screen.blit(load_object_image("posselect"),(self.ability_data[0]*BLOCK_SIZE,self.ability_data[1]*BLOCK_SIZE))
            #print "OK"


    def draw_status_line(self,y,map):
        #s = str( self.current_round ) + "/" + str(len(self.maps)-1) + u". kör"
        round_num = 0
        if self.current_round < len(self.maps):
            round_num = self.maps[self.current_round].round_number
        else:
            round_num = self.current_round - len(self.maps) + self.maps[-1].round_number +1
        s = str( round_num ) + "/" + str(self.maps[-1].round_number) + u". kör"
        if self.mode == MOVIE_MODE:
            s += u" (mozi)"
        elif self.current_round < len(self.maps)-1:
            s += u" (múlt)"
        elif self.current_round == len(self.maps)-1:
            s += u" (jelen)"
        else:
            s += u" (terv)"
        text = font.render(s,1, (0,0,0))
        textpos = text.get_rect(left=0, top=y)
        screen.blit(text, textpos)
        if self.check(sel = True, own = True,status = False) == False:
            x = textpos.width
            ob = [o for o in map.objects if o.id == self.chosen_object][0]
            s = u", Lépések: "
            text = font.render(s,1, (0,0,0))
            textpos = text.get_rect(left=x, top=y)
            x += textpos.width
            screen.blit(text, textpos)
            first = True
            i = 0;
            for step in ob.future_steps:
                s = (1-first)*", "+lang.ability(step)
                first = False
                text = font.render(s,1, (0+250*(i == self.active_step()),0,0))
                if (i == self.active_step()) and step.find(')') > -1 and step.find('(') > -1 and all(['0'<=c<='9' or c==' ' or c==',' for c in step[step.find('(')+1 : step.find(')')]]):
                    pos = eval('['+step[step.find('(')+1 : step.find(')')]+']')
                    if len(pos) == 2:
                        screen.blit(load_object_image("posselect"),(pos[0]*BLOCK_SIZE,pos[1]*BLOCK_SIZE))
                textpos = text.get_rect(left=x, top=y)
                x += textpos.width
                screen.blit(text, textpos)
                i+=1
        y += textpos.height
        text = font.render(self.status_line,1, (0,0,250))
        textpos = text.get_rect(left=0, top=y)
        screen.blit(text, textpos)
        return y+textpos.height

    def draw_Vakarcs_data(self,x,y,map):
        if self.player_info != None:
            act = self.player_info[min(self.current_round,len(self.maps)-1)]
#            for s in [ u"Pénz: " + str(act.money) + ("+"+str((self.current_round-len(self.maps)+1)*act.money_per_round))*(self.current_round >= len(self.maps)) ]:
#                text = font.render(s,1, (0,0,0))
            text = font.render(u"Pénz: " + str(act.money) + u"   uralt terület: "+str(int(act.rate*100))+"%",1, (0,0,0))
            textpos = text.get_rect(left=x, top=y)
            screen.blit(text, textpos)
            y += textpos.height

        if len([o for o in map.objects if o.id == self.chosen_object]):
            ob = [o for o in map.objects if o.id == self.chosen_object][0]
            for s in [ "Vakarcs: "+lang.type(ob.type) ] + [ lang.property(q)+": "+str(ob.prop[q]) for q in ob.prop if ob.prop[q] != 0] + [u"Képességek:"]:
                text = font.render(s,1, (0,0,0))
                textpos = text.get_rect(left=x, top=y)
                screen.blit(text, textpos)
                y += textpos.height
            i = 1
            for ab in ob.abilities:
                if ab.name == "move":
                    s = u"Nyilak: mozgás"
                else:
                    s = str(i) + ": " + lang.long_ability(ab.name,ab.data)
                    i+=1
                    if ab.type == "dir":
                        s += u"(irány)"
                    elif ab.type == "pos":
                        s += u"(pozíció)"
                text = font.render(s,1, (0,0,0))
                textpos = text.get_rect(left=x, top=y)
                screen.blit(text, textpos)
                y += textpos.height
                if ab.type == "shop":
                    for d in ab.data:
                        s = "  " + str(d[0])+u" pénzért " + lang.type(d[1])
                        text = font.render(s,1, (0,0,0))
                        textpos = text.get_rect(left=x, top=y)
                        screen.blit(text, textpos)
                        y += textpos.height
        return y

    def draw_stack(self,x,y,map):
        if self.check(): return
        s = u"Függőben lévő varázslatok:"
        text = font.render(s,1, (0,0,0))
        textpos = text.get_rect(left=x, top=y)
        screen.blit(text, textpos)
        y += textpos.height
        for i,st in enumerate(map.stack):
            if st[0][0]-(self.current_round-len(self.maps)+1)*(self.current_round >= len(self.maps))>=0:
                s = chr(97+i)+": "+str(st[0][0]-(self.current_round-len(self.maps)+1)*(self.current_round >= len(self.maps)))+u" kör múlva: "+lang.stack(st)
                if len(st[2]) == 2:
                    text = font.render(chr(97+i), 1, (255, 0,0))
                    textpos = text.get_rect(centerx=BLOCK_SIZE*st[2][0]+BLOCK_SIZE/2,centery=BLOCK_SIZE*st[2][1]+BLOCK_SIZE/2)
                    screen.blit(text, textpos)
                text = font.render(s,1, (0,0,0))
                textpos = text.get_rect(left=x, top=y)
                screen.blit(text, textpos)
                y += textpos.height
        return y

    def draw_autoshop(self,x,y,map):
        if self.check(): return
        text = font.render(u"Automatikus vásárlás, amint összejött a pénz:",1, (0,0,0))
        textpos = text.get_rect(left=x, top=y)
        screen.blit(text, textpos)
        y += textpos.height
        for i,s in enumerate(self.maps[-1].shop):
            text = font.render(u"F"+str(i+1)+u": " + str(s[0]) + u" pénzenként 1 "+lang.type(s[1])+u" vásárlása",1, ((i==self.autoshop)*250+0,0,0))
            textpos = text.get_rect(left=x, top=y)
            screen.blit(text, textpos)
            y += textpos.height
        return y

    def draw_help(self,x,y,map):
        for s in [u"Segítség:",\
                u"PgUp, PgDn, [, ]: idő tekergetése",\
                u"Backspace: jelenhez ugrás",\
                u"nyilak: mozgatás",\
                u"1-9: képességek",\
                u"F1-F9: automatikus vásárlás beállítása",\
                u"F12: kész vagyunk, küldheti a szervernek a lépéseinket",\
                u"Enter szöveg Enter: chat a többiekkel",\
                u"Esc: kilép"]:
            text = font.render(s,1, (0,0,0))
            textpos = text.get_rect(left=x, top=y)
            screen.blit(text, textpos)
            y += textpos.height
        return y

    def make_step(self,s):
        if self.client.ready:
            self.client.ready = False
            self.client.SendNickMess()
        if self.check(sel = True, own = True, past_change = True,too_many_steps = True): return
        selected_object = [o for o in self.maps[-1].objects if o.id == self.chosen_object]
        if len([a for a in selected_object[0].abilities if a.name == "move"]):
            selected_object[0].future_steps[self.current_round - len(self.maps) +1] = s
            self.current_round += 1
        else:
            self.status_line = u"A kijelölt vakarccsal nem tudsz lépni."

    def make_ability(self,i):
        if self.client.ready:
            self.client.ready = False
            self.client.SendNickMess()
        if self.check(sel = True, own = True, past_change = True,too_many_steps = True): return
        selected_object = [o for o in self.maps[-1].objects if o.id == self.chosen_object][0]
        list = [ab for ab in selected_object.abilities if ab.name != "move"]
        if i < len(list):
            if list[i].type == "static":
                selected_object.future_steps[self.current_round - len(self.maps) +1] = list[i].name+str("()")
                self.current_round += 1
            elif list[i].type == "pos":
                self.mode = ABILITY_MODE
                self.ability_mode = ABILITY_POS
                self.ability_data = selected_object.pos
                self.selected_ability = list[i]
                self.status_line = u"Kiválasztott képesség: "+lang.ability(list[i].name)+u". A nyilakkal és az ENTERrel adj meg egy mezőt!"
            elif list[i].type == "dir":
                self.mode = ABILITY_MODE
                self.ability_mode = ABILITY_DIR
                self.selected_ability = list[i]
                self.status_line = u"Kiválasztott képesség: "+lang.ability(list[i].name)+u". A nyilakkal adj meg egy irányt!"
            elif list[i].type == "shop":
                ch = menu_choose(u"Melyik vakarcsot válsztod?",[str(v[0]) + u" pénzért " + lang.type(v[1]) for v in list[i].data])
                if ch >= 0:
                    selected_object.future_steps[self.current_round - len(self.maps) +1] = list[i].name+"("+list[i].data[ch][1]+")"
                    self.current_round += 1
                    #print ch
        else:
            self.status_line = u"Nincs ilyen képesség."


    def next_vakarcs(self,map):
        try:
            ids = [o.id for o in map.objects if o.owner == self.client.id]
            if self.chosen_object in ids:
                self.chosen_object = ids[ (ids.index(self.chosen_object)+1) % len(ids) ]
            else:
                self.chosen_object = ids[0]
        except :
            pass

    def previous_vakarcs(self,map):
        try:
            ids = [o.id for o in map.objects if o.owner == self.client.id]
            if self.chosen_object in ids:
                self.chosen_object = ids[ (ids.index(self.chosen_object)-1) % len(ids) ]
            else:
                self.chosen_object = ids[0]
        except :
            pass

    def back_in_time(self):
        if self.check(): return
        if self.current_round == 0:
            self.status_line = u"Ez a kezdő állapot, korábbra nem mehetsz vissza."; return
        self.current_round -= 1
        #if self.current_round < len(self.maps)-1:
        #    self.status_line = u"Ez a múlt.";
        #elif self.current_round == len(self.maps)-1:
        #    self.status_line = u"Ez a jelen.";
        #else:
        #    self.status_line = u"Ezek csak tervek a jövőre.";
        if self.current_round < len(self.maps) and self.maps[self.current_round].message != "":
            self.status_line = self.maps[self.current_round].message


    def for_in_time(self):
        if len(self.maps) and self.maps[min(self.current_round,len(self.maps)-1)].winners != []:
            return
        if self.check(too_many_steps = True): return
        self.current_round += 1
        #if self.current_round < len(self.maps)-1:
        #    self.status_line = u"Ez a múlt.";
        #elif self.current_round == len(self.maps)-1:
        #    self.status_line = u"Ez a jelen.";
        #else:
        #    self.status_line = u"Ezek csak tervek a jövőre.";
        if self.current_round < len(self.maps) and self.maps[self.current_round].message != "":
            self.status_line = self.maps[self.current_round].message

    def sync(self):
        self.redraw = True
        self.status_line = u"Mindenki lépett, új kör!"
        self.current_round = self.last_maps_size
        self.last_maps_size = len(self.maps)
        self.mode = MOVIE_MODE
        if len(self.maps)==1:
            self.next_vakarcs(self.maps[0])

    def run(self):
        chat = False
        map = None
        while 1:
            pygame.display.flip()
            if self.server != None:
                self.server.Pump()
            self.client.Loop()

            for event in pygame.event.get():
                if event.type == KEYDOWN and self.maps != [] and K_F1<=event.key<K_F1+len(self.maps[-1].shop):
                    self.autoshop = event.key-K_F1
                    if self.client.ready:
                        self.client.ready = False
                        self.client.SendNickMess()
                if event.type in [KEYDOWN, MOUSEBUTTONDOWN]:
                    self.redraw = True
                    self.status_line = ""
                if event.type == MOUSEBUTTONDOWN:
                    new_sel = [b[1] for b in self.bounding_boxes if b[0].collidepoint(event.pos)]
                    if new_sel != []:
                        self.chosen_object = new_sel[0]
                if event.type == QUIT:
                    sys.exit(0)
                if chat:
                    if event.type == KEYDOWN and event.key in [K_ESCAPE,K_RETURN]:
                        chat = False
                    elif event.type == KEYDOWN and event.key == K_BACKSPACE:
                        self.client.message = self.client.message[:-1]
                        self.client.SendNickMess()
                    elif event.type == KEYDOWN and event.key == K_SPACE:
                        self.client.message += " "
                        self.client.SendNickMess()
                    elif event.type == KEYDOWN and len(pygame.key.name(event.key))==1:
                        if pygame.key.get_mods() & KMOD_SHIFT:
                            self.client.message += pygame.key.name(event.key).upper()
                        else:
                            self.client.message += pygame.key.name(event.key)
                        self.client.SendNickMess()
                else: #chat == False
                    if self.mode == NORMAL_MODE:
                        if event.type == KEYDOWN and event.key == K_ESCAPE:
                            if input_text(question=u"Biztos kilépsz?",only_one_key=True) in ["y","Y","i","I"]:
                                sys.exit(0)
                            #sys.exit(0) # Temporary
                        elif event.type == KEYDOWN and event.key == K_TAB:
                            if pygame.key.get_mods() & KMOD_SHIFT:
                                self.previous_vakarcs(map)
                            else:
                                self.next_vakarcs(map)
                        elif event.type == KEYDOWN and event.key == K_RETURN:
                            chat = True
                            self.client.message = ""
                            self.client.SendNickMess()
                        elif event.type == KEYDOWN and event.key == K_BACKSPACE:
                            self.current_round = len(self.maps)-1
                            self.status_line = u"Ez a jelen.";
                        elif event.type == KEYDOWN and event.key in [K_RIGHT,K_l]:
                            self.make_step("move(0)")
                        elif event.type == KEYDOWN and event.key in [K_UP,K_k]:
                            self.make_step("move(1)")
                        elif event.type == KEYDOWN and event.key in [K_LEFT,K_h]:
                            self.make_step("move(2)")
                        elif event.type == KEYDOWN and event.key in [K_DOWN,K_j]:
                            self.make_step("move(3)")
                        elif event.type == KEYDOWN and event.key in [K_PAGEUP,K_LEFTBRACKET]:
                            self.back_in_time()
                        elif event.type == KEYDOWN and event.key in [K_PAGEDOWN,K_RIGHTBRACKET]:
                            self.for_in_time()
                        elif event.type == KEYDOWN and event.key in [K_F12]:
                            if self.maps != []:
                                self.client.ready = 1-self.client.ready
                                if self.client.ready:
                                    self.client.SendMap()
                                self.client.SendNickMess()
                        elif event.type == KEYDOWN and event.key in [K_1,K_2,K_3,K_4,K_5,K_6,K_7,K_8,K_9]:
                            i = [K_1,K_2,K_3,K_4,K_5,K_6,K_7,K_8,K_9].index(event.key)
                            self.make_ability(i)
                        elif event.type == KEYDOWN and event.key == K_SPACE: # to do nothing
                            if self.check(sel = True, own = True, past_change = True,too_many_steps = True): return
                            selected_object = [o for o in self.maps[-1].objects if o.id == self.chosen_object][0]
                            selected_object.future_steps[self.current_round - len(self.maps) +1] = ""
                            self.current_round += 1
                    elif self.mode == ABILITY_MODE: #ABILITY MODE
                        if event.type == KEYDOWN and event.key == K_ESCAPE:
                            self.mode = NORMAL_MODE
                        elif event.type == KEYDOWN and event.key in [K_1,K_2,K_3,K_4,K_5,K_6,K_7,K_8,K_9]:
                            i = [K_1,K_2,K_3,K_4,K_5,K_6,K_7,K_8,K_9].index(event.key)
                            self.make_ability(i)
                        elif event.type == KEYDOWN and event.key == K_TAB:
                            self.mode = NORMAL_MODE
                            if pygame.key.get_mods() & KMOD_SHIFT:
                                self.previous_vakarcs(map)
                            else:
                                self.next_vakarcs(map)
                        elif event.type == KEYDOWN and event.key == K_RETURN:
                            if self.ability_mode == ABILITY_DIR:
                                self.status_line = u"Egy irányt adj meg a nyilakkal!"
                            elif self.ability_mode == ABILITY_POS:
                                if self.check(sel = True, own = True, past_change = True,too_many_steps = True):
                                    self.mode = NORMAL_MODE
                                    return
                                selected_object = [o for o in self.maps[-1].objects if o.id == self.chosen_object][0]
                                #print self.selected_ability.name+"("+str( self.ability_data )[1:-1]+")"
                                selected_object.future_steps[self.current_round - len(self.maps) +1] = self.selected_ability.name+"("+str( self.ability_data )[1:-1]+")"
                                self.current_round += 1
                                self.mode=NORMAL_MODE
                        dir = -1
                        if event.type == KEYDOWN and event.key in [K_RIGHT,K_l]:
                            dir = 0
                        elif event.type == KEYDOWN and event.key in [K_UP,K_k]:
                            dir = 1
                        elif event.type == KEYDOWN and event.key in [K_LEFT,K_h]:
                            dir = 2
                        elif event.type == KEYDOWN and event.key in [K_DOWN,K_j]:
                            dir = 3
                        if dir >= 0:
                            if self.ability_mode == ABILITY_POS:
                                self.ability_data = [self.ability_data[0]+(dir==0)-(dir==2),self.ability_data[1]+(dir==3)-(dir==1)]
                            elif self.ability_mode == ABILITY_DIR:
                                if self.check(sel = True, own = True, past_change = True,too_many_steps = True): return
                                selected_object = [o for o in self.maps[-1].objects if o.id == self.chosen_object][0]
                                selected_object.future_steps[self.current_round - len(self.maps) +1] = self.selected_ability.name+"("+str( dir )+")"
                                self.current_round += 1
                                self.mode = NORMAL_MODE
                    elif self.mode == MOVIE_MODE: #MOVIE_MODE
                        if event.type in [KEYDOWN,MOUSEBUTTONDOWN]:
                            self.mode = NORMAL_MODE

            if self.mode == MOVIE_MODE: #MOVIE_MODE
                #print "MOVIE_MODE"
                self.movie_count += 1
                if self.movie_count > 5:
                    self.movie_count = 0
                    if self.current_round >= len(self.maps)-1:
                        self.mode = NORMAL_MODE
                    else:
                        self.for_in_time()
                    self.redraw = True


            if self.redraw:
                if len(self.maps):
                    map = self.maps[min(self.current_round,len(self.maps)-1)]
                screen.blit(self.background, (0, 0))
                
                y = 0
                #draw map here
                if map != None:
                    self.draw_map(map)
                    
                    y=map.size()[1]*BLOCK_SIZE

                    y = self.draw_status_line(y,map)
                    yy= self.draw_Vakarcs_data(map.size()[0]*BLOCK_SIZE,0,map)
                    self.draw_stack(map.size()[0]*BLOCK_SIZE,max(yy,map.size()[1]/4*BLOCK_SIZE),map)
                    yyy=self.draw_autoshop(map.size()[0]*BLOCK_SIZE,max(yy,2*map.size()[1]/4*BLOCK_SIZE),map)
                    yyy+=7
                    self.draw_help(map.size()[0]*BLOCK_SIZE,yyy,map)
                    if (map.winners != []):
                        w = [n for i,n in enumerate(self.client.players) if self.client.ids[i] in map.winners]
                        text = bigfont.render(u"Győztes: "+str(w)[1:-1], 1, (10, 10, 10))
                        textpos = text.get_rect(centerx=background.get_width()/2,centery=background.get_height()/2)
                        screen.blit(text, textpos)
                self.status_line = ""

                for name,ready,mess,id in zip(self.client.players,self.client.readys,self.client.messages,self.client.ids):
                    text = font.render(name+u" (kész) "*int(ready)+": "+mess, 1, (10+chat*(self.client.id==id)*200, 10, 10))
                    textpos = text.get_rect(left=0, top=y)
                    screen.blit(text, textpos)
                    y+=textpos.height
                self.redraw = False
    

            clock.tick(FPS)

